"""My Nagios plugin wrapper."""
# pylint: disable=logging-fstring-interpolation
__version__ = "0.1.6"

import argparse
import logging
import os.path
import socket
import sys

import requests

from mnpw import nagios

HOSTNAME = socket.gethostname()


def notify(message):
    """Send a notification."""
    for _ in range(3):
        r = requests.post(  # pylint: disable=invalid-name
            "https://notify.shore.co.il/send", params={"message": message}
        )
        if r.ok:
            return
        logging.warning(
            f"Failed to send notification: {r.reason}, will retry."
        )
        logging.error("Failed to send notification.")


def notify_wrapper(check):
    """mnpw specific notifications.

    The above notify function is generic, this function is more specific to the
    notifications mnpw sends.
    """
    if check.Output is None or check.Output.strip() == "":
        notify(
            f"[{HOSTNAME}] {os.path.basename(check.Command)} returned {check.ExitCode}."  # noqa: E501
        )
    else:
        notify(f"[{HOSTNAME}] {check.Output}")


def main():  # noqa: MC0001
    """Main entrypoint."""
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("command", type=str, help="Plugin command")
    parser.add_argument(
        "arguments",
        type=str,
        nargs="*",
        help="Command arguments",
    )
    parser.add_argument(
        "-v", "--verbose", help="Verbose output", action="store_true"
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"mnpw version {__version__}",
    )
    parser.add_argument(
        "-c",
        "--chain",
        help="Echo the check output and exit with the same exit code, allow for chaining with plugin handlers.",  # noqa: E501
        action="store_true",
    )
    parser.add_argument(
        "-d", "--dry-run", help="Dry-run, don't notify", action="store_true"
    )
    parser.add_argument(
        "-q",
        "--quiet",
        "--silent",
        help="No output, except for errors.",
        action="store_true",
    )
    parser.add_argument(
        "-w",
        "--warn",
        help="Notify when the check status is WARNING.",
        action="store_true",
    )
    parser.add_argument(
        "-u",
        "--unknown",
        help="Notify when the check status is UNKNOWN.",
        action="store_true",
    )
    parser.add_argument(
        "-e",
        "--errors",
        help="Notify on check errors.",
        action="store_true",
    )
    parser.add_argument(
        "-t",
        "--timeout",
        help=f"Command timeout (in seconds), defaults to {nagios.DEFAULT_TIMEOUT}",  # noqa: E501
        type=int,
        default=nagios.DEFAULT_TIMEOUT,
    )
    args = parser.parse_args()

    log_level = logging.WARNING  # Default level.
    if args.verbose and args.quiet:
        parser.error("Can't specify verbose and quiet output together.")
    if args.verbose:
        log_level = logging.INFO
    if args.quiet:
        log_level = logging.CRITICAL
    logging.basicConfig(format="%(message)s", level=log_level)

    check = nagios.Check(args.command, args.arguments)
    try:
        check.run(args.timeout)
    except Exception as ex:  # pylint: disable=broad-except
        if args.errors and not args.dry_run:
            notify(f"[{HOSTNAME}] Check {args.command} failed: {ex}.")
        parser.error(str(ex))

    if check.ExitCode == nagios.NagiosCode.OK:
        logging.info("Check status is OK.")
    elif check.ExitCode == nagios.NagiosCode.WARNING:
        logging.info("Check status is WARNING.")
        if args.warn and not args.dry_run:
            notify_wrapper(check)
    elif check.ExitCode == nagios.NagiosCode.CRITICAL:
        logging.info("Check status is CRITICAL.")
        if not args.dry_run:
            notify_wrapper(check)
    elif check.ExitCode == nagios.NagiosCode.UNKNOWN:
        logging.info("Check status is UNKNOWN.")
        if args.unknown and not args.dry_run:
            notify_wrapper(check)
    else:
        logging.info(
            f"Check status is invalid for a Nagios plugin ({check.ExitCode})."
        )
        if args.errors:
            notify(f"[{HOSTNAME}] Check {args.command} status is invalid.")

    if args.chain:
        print(check._stdout)  # pylint: disable=protected-access
        sys.exit(check.ExitCode)


if __name__ == "__main__":
    main()
